
#import "SimpleRESTRequest.h"

@implementation SimpleRESTRequest

- (id) init
{
	return [self initWithBaseURL: nil 
 	                   targetURL: nil 
                      parameters: nil];

}

- (id) initWithBaseURL: (NSString *) baseURL
{
	return [self initWithBaseURL: baseURL
 	                   targetURL: nil 
                      parameters: nil];

}

- (id) initWithBaseURL: (NSString *) baseURL targetURL: (NSString *) targetURL 
{
	return [self initWithBaseURL: baseURL
 	                   targetURL: targetURL
                      parameters: nil];
}

- (id) initWithBaseURL: (NSString *) newBaseURL 
		     targetURL: (NSString *) newTargetURL
            parameters: (NSDictionary *) newParameters
{
	self = [super init];

	if(self)
	{
		_baseURL = newBaseURL;
		_targetURL = newTargetURL;
		_parameters = newParameters;
	}

	return self;
}

- (void) dealloc
{
	[_baseURL release];
	[_targetURL release];
	[_parameters release];
	[super dealloc];
}

- (NSString *) baseURL
{
	return _baseURL;
}

- (void) setBaseURL: (NSString *) baseURL 
{
	if(_baseURL) 
	{
		[_baseURL release];
	}
	_baseURL = baseURL;
	[_baseURL retain];
}


- (NSString *) targetURL 
{
	return _targetURL;
}

- (void) setTargetURL: (NSString *) targetURL 
{
	if(_targetURL) 
	{
		[_targetURL release];
	}
	_targetURL = targetURL;
	[_targetURL retain];
}

- (NSDictionary *) parameters 
{
	return _parameters;
}

- (void) setParameters: (NSDictionary *) parameters 
{
	if(_parameters)
	{
		[_parameters release];
	}
	_parameters = parameters;
	[_parameters retain];
}

- (void) start
{
	if(_running) return;

	_running = YES;

	if(_baseURL == nil && _targetURL == nil)
	{
		NSLog(@"No URL defined for request");
		return;
	}

	NSString *urlString = [NSString stringWithFormat:@"%@/%@", _baseURL, _targetURL];
	NSURL *url = [NSURL URLWithString: urlString];
    _connection = [NSURLConnection connectionWithRequest: [NSURLRequest requestWithURL: url] 
  											    delegate: self];	
	if(_connection)
 	{
		if(_data) [_data release];
		_data = [[NSMutableData alloc] init];
	    NSLog(@"starting request %@", [url absoluteString]);
	    [_connection start];
	}
}

- (void) stop
{
	_running = NO;
	[_data release];
	[_connection cancel];
	[_connection release];
}

#pragma mark - NSURLRequest methods

- (void) connectionDidFinishLoading: (NSURLConnection *) connection
{
	NSLog(@"finished");
 
	NSError *error = nil;
    id json = [NSJSONSerialization JSONObjectWithData: _data options: 0 error: &error];

	if(error)
	{
		NSLog(@"Error parsing JSON %@", [error localizedDescription]);
	}
	else
	{
		NSDictionary *userInfo = [NSDictionary dictionaryWithObject: json forKey: SimpleRESTRequestJSONKey];
		[[NSNotificationCenter defaultCenter] postNotificationName: SimpleRESTRequestDidSucceedNotification
 												        object: self
											          userInfo: userInfo]; 
	}
}

- (void) connection: (NSURLConnection *) connection didReceiveData: (NSData *) data
{
	NSLog(@"did receive data");
	[_data appendData: data];
}

- (void) connection: (NSURLConnection *) connection didReceiveResponse: (NSURLResponse *) response 
{
	NSLog(@"response");
}

- (void) connection: (NSURLConnection *) connection didFailWithError: (NSError *) error
{
	_running = NO;
	[_data release];

	NSLog(@"connection failed");

    NSDictionary *userInfo = [NSDictionary dictionaryWithObject: error forKey: SimpleRESTRequestFailureKey];
    [[NSNotificationCenter defaultCenter] postNotificationName: SimpleRESTRequestDidFailNotification
 												        object: self
											          userInfo: userInfo]; 
}

@end

NSString * const SimpleRESTRequestDidSucceedNotification = @"SimpleRESTRequestDidSucceed";
NSString * const SimpleRESTRequestJSONKey = @"SimpleRESTRequestJSONKey";
NSString * const SimpleRESTRequestDidFailNotification = @"SimpleRESTRequestDidFail";
NSString * const SimpleRESTRequestFailureKey = @"SimpleRESTRequestFailureKey";

