//
//  ViewController.m
//  IMDB
//
//  Created by Brennan Cleveland on 6/24/12.
//  Copyright (c) 2012 Livefront. All rights reserved.
//

#import "SimpleRESTRequest.h"
#import "ViewController.h"

@interface ViewController ()
{
    NSMutableData *_data;
}

@end

@implementation ViewController

@synthesize toolbar;
@synthesize imageView;
@synthesize busyIndicator;

- (void) viewDidLoad
{
    [[NSNotificationCenter defaultCenter] addObserver: self
                                             selector: @selector(requestSucceeded:) 
                                                 name: SimpleRESTRequestDidSucceedNotification 
                                               object: nil];
    
    [[NSNotificationCenter defaultCenter] addObserver: self
                                             selector: @selector(requestFailed:) 
                                                 name: SimpleRESTRequestDidFailNotification 
                                               object: nil];

    [super viewDidLoad];
}

- (void) viewDidUnload
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    self.busyIndicator = nil;
    self.imageView = nil;
    self.toolbar = nil;
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

#pragma mark - UISearchBarDelegate methods

- (void) searchBarSearchButtonClicked:(UISearchBar *)searchBar
{
    [searchBar resignFirstResponder];
    NSString *title = [searchBar.text stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding];
    SimpleRESTRequest *request = [[SimpleRESTRequest alloc] init];
    request.baseURL =  @"http://www.imdbapi.com";
    request.targetURL = [NSString stringWithFormat:@"?t=%@", title];
    [request start];
    
    [self toggleView:self.imageView visible:NO];
    self.busyIndicator.hidden = NO;
}

#pragma mark - NSURLConnection delegate methods

- (void) connectionDidFinishLoading: (NSURLConnection *) connection
{
    [self toggleView:self.imageView visible:YES];
    self.busyIndicator.hidden = YES;
    self.imageView.image = [UIImage imageWithData:_data];
    [_data release];
    _data = nil;
}

- (void) connection: (NSURLConnection *) connection didReceiveData: (NSData *) data
{
	NSLog(@"did receive data");
	[_data appendData: data];
}

- (void) connection: (NSURLConnection *) connection didReceiveResponse: (NSURLResponse *) response 
{
	NSLog(@"response");
}

- (void) connection: (NSURLConnection *) connection didFailWithError: (NSError *) error
{
	[_data release];
    _data = nil;
    self.busyIndicator.hidden = YES;
    
    [self showErrorMessage:[error localizedDescription]];
}

#pragma mark - IBActions
     
- (IBAction) didTapButton:(id)sender
{
    [self.view endEditing:YES];
}

#pragma mark - private methods

- (void) requestSucceeded: (NSNotification *) notification
{
	NSDictionary *data = [[notification userInfo] valueForKey: SimpleRESTRequestJSONKey];
	NSLog(@"%@", [data description]);
    
    NSString *imageURL = [data valueForKey:@"Poster"];
    
    if(imageURL)
    {
        _data = [[NSMutableData alloc] init];
        NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:imageURL]];
        NSURLConnection *connection = [NSURLConnection connectionWithRequest:request delegate:self];
        
        [connection start];
    }
    else
    {
        [self showErrorMessage:@"No Poster Available!"];
    }
}

- (void) requestFailed: (NSNotification *) notification
{
	NSError *error = [[notification userInfo] valueForKey: SimpleRESTRequestFailureKey];
	NSLog(@"%@", [error localizedDescription]);
    
    self.busyIndicator.hidden = YES;
    
    [self showErrorMessage: [error localizedDescription]];
}

- (void) showErrorMessage: (NSString *) message
{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Whoops!" 
                                                    message:message 
                                                   delegate:nil 
                                          cancelButtonTitle:@"Ok" 
                                          otherButtonTitles: nil];
    [alert show];
    [alert release];
}

- (void) toggleView: (UIView *) view visible: (BOOL) visible
{
    if(view == nil) return;
    
    [UIView beginAnimations:@"toggle_view" context:nil];
    [UIView setAnimationDuration:0.3f];
    
    if(visible)
    {
        view.alpha = 1.0;
    }
    else 
    {
        view.alpha = 0.0;
    }
    
    [UIView commitAnimations];
    
}

@end
